/*{
    "DESCRIPTION": "Every pixel is blue.",
    "CREDIT": "pyvvisf example",
    "ISFVSN": "2.0",
    "CATEGORIES": ["Generator"],
    "INPUTS": []
}*/
void main() {
    gl_FragColor = vec4(0.0, 0.5, 1.0, 1.0); // blue
} 